# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse_lazy, reverse
from django.http.response import HttpResponseRedirect
from django.views.generic import CreateView, DetailView, ListView
from .models import Datum, User
from .util import generate_day_labels, generate_datum_frequency, generate_usage, generate_unique_users


class DatumListView(ListView):
    model = Datum

    def get_context_data(self, **kwargs):
        context = super(DatumListView, self).get_context_data(**kwargs)
        context['activity'] = generate_unique_users()
        context['labels'] = generate_day_labels()
        return context


class DatumCreateView(CreateView):
    model = Datum
    fields = ['user', 'name', 'value']
    success_url = reverse_lazy('datum_list')

    def post(self, request, *args, **kwargs):
        if 'uuid' in request.POST:
            uuid = request.POST['uuid']
            user, created = User.objects.get_or_create(uuid=uuid)
            Datum.objects.create(user=user, name=request.POST['name'], value=request.POST['value'])
            return HttpResponseRedirect(reverse('datum_list'))
        return super(DatumCreateView, self).post(self, request, *args, **kwargs)


class DatumDetailView(DetailView):
    model = Datum

    def get_context_data(self, **kwargs):
        context = super(DatumDetailView, self).get_context_data(**kwargs)
        context['datums'] = Datum.objects.filter(name=self.get_object().name)
        context['activity'] = generate_datum_frequency(self.get_object().name)
        context['labels'] = generate_day_labels()
        return context


class UserDetailView(DetailView):
    model = User

    def get_context_data(self, **kwargs):
        context = super(UserDetailView, self).get_context_data(**kwargs)
        context['datums'] = Datum.objects.filter(user=self.get_object())
        context['activity'] = generate_usage(Datum.objects.filter(user=self.get_object()))
        context['labels'] = generate_day_labels()
        return context



from django.contrib import admin
from .models import User, Datum


class UserAdmin(admin.ModelAdmin):
    list_display = ('id', 'uuid')
    search_fields = ('uuid',)
admin.site.register(User, UserAdmin)


class DatumAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'value', 'user', 'modified', 'created')
    search_fields = ('name', 'value', 'user')
admin.site.register(Datum, DatumAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-12-01 03:46
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('django_lair', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=datetime.datetime(2016, 12, 1, 3, 46, 35, 159850, tzinfo=utc)),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from model_utils.models import TimeStampedModel


@python_2_unicode_compatible
class User(models.Model):
    uuid = models.UUIDField()
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return str(self.uuid)


@python_2_unicode_compatible
class Datum(TimeStampedModel):
    user = models.ForeignKey(User)
    name = models.CharField(max_length=200, db_index=True)
    value = models.TextField()

    def __str__(self):
        return str(self.user) + ' ' + self.name + ' ' + self.value + ' ' + str(self.created)



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt
from . import views


urlpatterns = [
    url(
        regex="^datum/$",
        view=views.DatumListView.as_view(),
        name='datum_list',
    ),
    url(
        regex="^datum/create/$",
        view=csrf_exempt(views.DatumCreateView.as_view()),
        name='datum_create',
    ),
    url(
        regex="^datum/(?P<pk>\d+)/$",
        view=views.DatumDetailView.as_view(),
        name='datum_detail',
    ),
    url(
        regex="^user/(?P<pk>\d+)/$",
        view=views.UserDetailView.as_view(),
        name='user_detail',
    ),
]



# -*- coding: utf-8 -*-
from datetime import timedelta
from django.db.models import Q
from django.utils import timezone
from .models import Datum


""" From a list of datums, generate a list representing frequency """
def generate_usage(data, days=30):
    start_date = timezone.now() - timedelta(days=days - 1)
    data = data.filter(created__gt=start_date).order_by('created')
    data_usage = [0] * days
    last_day = start_date.day
    index = 0
    for datum in data:
        if last_day != datum.created.day:
            index += datum.created.day - last_day
            last_day = datum.created.day
        data_usage[index] += 1
    return data_usage


def generate_unique_users(days=30):
    counts = []
    for day in range(days):
        counts.append(Datum.objects.filter(created_on_day_q(day)).values('user').distinct().count())
    counts.reverse()
    return counts


def generate_datum_frequency(name, days=30):
    counts = []
    for day in range(days):
        counts.append(Datum.objects.filter(name=name).filter(created_on_day_q(day)).count())
    counts.reverse()
    return counts


def generate_day_labels(days=30):
    start_date = timezone.now() - timedelta(days=days - 1)
    return [(start_date + timedelta(i - 1)).day for i in range(days)]


def created_on_day_q(day):
    return Q(created__gt=timezone.now() - timedelta(days=day + 1)) &\
        Q(created__lt=timezone.now() - timedelta(days=day))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-30 04:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Datum',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('value', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField()),
            ],
        ),
        migrations.AddField(
            model_name='datum',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_lair.User'),
        ),
    ]
